/* PgSqlClient - ADO.NET Data Provider for PostgreSQL 7.4+
 * Copyright (c) 2003-2004 Carlos Guzman Alvarez
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Data;
using System.Text;

namespace PostgreSql.Data.PgSqlClient.DbSchema
{
	internal class PgFunctionPrivilegesSchema : PgAbstractDbSchema
	{
		#region Constructors

		public PgFunctionPrivilegesSchema() : base("Function_Privileges")
		{
		}

		#endregion

		#region Add Methods

		public override void AddTables()
		{
			AddTable("pg_proc");
		}

		public override void AddRestrictionColumns()
		{
			AddRestrictionColumn("pg_namespace.nspname"	, "FUNCTION_SCHEMA", null);
			AddRestrictionColumn("pg_proc.proname"		, "FUNCTION_NAME", null);
		}

		public override void AddDataColumns()
		{
			AddDataColumn("pg_proc.proacl", "PRIVILEGES");
		}

		public override void AddJoins()
		{
			AddJoin("left join", "pg_namespace"	, "pg_proc.pronamespace = pg_namespace.oid");
		}

		public override void AddOrderByColumns()
		{
			AddOrderBy("pg_namespace.nspname");
			AddOrderBy("pg_proc.proname");
		}

		public override void AddWhereFilters()
		{
		}

		#endregion

		#region Parse Methods

		public override object[] ParseRestrictions(object[] restrictions)
		{
			object[] parsed = restrictions;

			return parsed;
		}

		#endregion

		#region Overriden Methods

		public override DataTable GetDbSchemaTable(PgConnection connection, object[] restrictions)
		{
			DataTable tablesSchema = base.GetDbSchemaTable(connection, restrictions);
			DataTable privileges = getPrivilegesDataTable();

			privileges.BeginLoadData();

			foreach (DataRow row in tablesSchema.Rows)
			{
				if (row["PRIVILEGES"] != System.DBNull.Value)
				{
					PgPrivilege[] priv = DecodePrivileges((string[])row["PRIVILEGES"]);

					for (int i = 0; i < priv.Length; i++)
					{
						DataRow newRow = privileges.NewRow();

						newRow["FUNCTION_SCHEMA"]	= row["FUNCTION_SCHEMA"];
						newRow["FUNCTION_NAME"]		= row["FUNCTION_NAME"];
						newRow["USER_NAME"]			= priv[i].User;

						FillPrivileges(newRow, priv[i].Privileges);

						privileges.Rows.Add(newRow);
					}
				}
			}

			privileges.EndLoadData();

			return privileges;
		}

		private DataTable getPrivilegesDataTable()
		{
			DataTable privileges = new DataTable("Function_Privileges");

			privileges.BeginInit();

			privileges.Columns.Add("FUNCTION_SCHEMA", Type.GetType("System.String"));
			privileges.Columns.Add("FUNCTION_NAME"	, Type.GetType("System.String"));
			privileges.Columns.Add("USER_NAME"		, Type.GetType("System.String"));

			AddPrivilegesColumns(privileges);

			privileges.EndInit();

			return privileges;
		}

		#endregion

		#region Private Methods

		private string getVolatileExpression(string fieldName)
		{
			StringBuilder expression = new StringBuilder();

			expression.AppendFormat(" case {0} ", fieldName);
			expression.Append(" when 'i' THEN 'INMUTABLE'");
			expression.Append(" when 's' THEN 'STABLE'");
			expression.Append(" when 'v' THEN 'VOLATILE'");
			expression.Append(" END ");

			return expression.ToString();
		}

		#endregion
	}
}